@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2001-2005 Michael Terry
+Copyright (c) 2001-2007 Michael Terry
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "xpad-text-buffer.h"
 #include "xpad-text-view.h"
 #include "xpad-toolbar.h"
+#include "xpad-tray.h"
 
 G_DEFINE_TYPE(XpadPad, xpad_pad, GTK_TYPE_WINDOW)
 #define XPAD_PAD_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), XPAD_TYPE_PAD, XpadPadPrivate))
@@ -53,6 +54,8 @@ struct XpadPadPrivate
 	GtkWidget *toolbar;
 	guint toolbar_timeout;
 	gint toolbar_height;
+	gboolean toolbar_expanded;
+	gboolean toolbar_pad_resized;
 	
 	/* properties window */
 	GtkWidget *properties;
@@ -77,7 +80,7 @@ enum
   LAST_PROP
 };
 
-static void load_info (XpadPad *pad);
+static void load_info (XpadPad *pad, gboolean *show);
 static void save_info (XpadPad *pad);
 static void load_content (XpadPad *pad);
 static void save_content (XpadPad *pad);
@@ -106,11 +109,11 @@ static void xpad_pad_show_toolbar (XpadPad *pad);
 static void xpad_pad_popup (XpadPad *pad, GdkEventButton *event);
 static void xpad_pad_spawn (XpadPad *pad);
 static void xpad_pad_clear (XpadPad *pad);
-static void xpad_pad_close (XpadPad *pad);
 static void xpad_pad_delete (XpadPad *pad);
 static void xpad_pad_open_properties (XpadPad *pad);
 static void xpad_pad_open_preferences (XpadPad *pad);
 static void xpad_pad_quit (XpadPad *pad);
+static void xpad_pad_close_all (XpadPad *pad);
 static void xpad_pad_sync_title (XpadPad *pad);
 static void xpad_pad_set_group (XpadPad *pad, XpadPadGroup *group);
 static gboolean xpad_pad_leave_notify_event (GtkWidget *pad, GdkEventCrossing *event);
@@ -128,13 +131,47 @@ xpad_pad_new (XpadPadGroup *group)
 }
 
 GtkWidget *
-xpad_pad_new_with_info (XpadPadGroup *group, const gchar *info_filename)
+xpad_pad_new_with_info (XpadPadGroup *group, const gchar *info_filename, gboolean *show)
 {
 	GtkWidget *pad = GTK_WIDGET (g_object_new (XPAD_TYPE_PAD, "group", group, NULL));
 	
 	XPAD_PAD (pad)->priv->infoname = g_strdup (info_filename);
-	load_info (XPAD_PAD (pad));
+	load_info (XPAD_PAD (pad), show);
 	load_content (XPAD_PAD (pad));
+	gtk_window_set_role (GTK_WINDOW (pad), XPAD_PAD (pad)->priv->infoname);
+	
+	return pad;
+}
+
+GtkWidget *
+xpad_pad_new_from_file (XpadPadGroup *group, const gchar *filename)
+{
+	GtkWidget *pad = NULL;
+	gchar *content;
+	
+	content = fio_get_file (filename);
+	
+	if (!content)
+	{
+		gchar *usertext = g_strdup_printf (_("Could not read file %s."), filename);
+		xpad_app_error (NULL, usertext, NULL);
+		g_free (usertext);
+	}
+	else
+	{
+		GtkTextBuffer *buffer;
+		
+		pad = GTK_WIDGET (g_object_new (XPAD_TYPE_PAD, "group", group, NULL));
+		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (XPAD_PAD (pad)->priv->textview));
+		
+		g_signal_handlers_block_by_func (buffer, xpad_pad_text_changed, pad);
+		
+		xpad_text_buffer_set_text_with_tags (XPAD_TEXT_BUFFER (buffer), content ? content : "");
+		g_free (content);
+		
+		g_signal_handlers_unblock_by_func (buffer, xpad_pad_text_changed, pad);
+		xpad_pad_text_changed(XPAD_PAD(pad), buffer);
+	}
 	
 	return pad;
 }
@@ -186,12 +223,14 @@ xpad_pad_init (XpadPad *pad)
 	pad->priv->height = xpad_settings_get_height (xpad_settings ());
 	pad->priv->infoname = NULL;
 	pad->priv->contentname = NULL;
-	pad->priv->sticky = FALSE;
+	pad->priv->sticky = xpad_settings_get_sticky (xpad_settings ());
 	pad->priv->textview = NULL;
 	pad->priv->scrollbar = NULL;
 	pad->priv->toolbar = NULL;
 	pad->priv->toolbar_timeout = 0;
 	pad->priv->toolbar_height = 0;
+	pad->priv->toolbar_expanded = FALSE;
+	pad->priv->toolbar_pad_resized = TRUE;
 	pad->priv->properties = NULL;
 	pad->priv->group = NULL;
 	
@@ -217,6 +256,9 @@ xpad_pad_init (XpadPad *pad)
 	g_object_unref (G_OBJECT (accel_group));
 	pad->priv->menu = menu_get_popup_no_highlight (pad, accel_group);
 	pad->priv->highlight_menu = menu_get_popup_highlight (pad, accel_group);
+	gtk_accel_group_connect (accel_group, GDK_Q, GDK_CONTROL_MASK, 0,
+	                         g_cclosure_new_swap (G_CALLBACK (xpad_pad_quit), pad, NULL));
+
 	
 	vbox = GTK_WIDGET (g_object_new (GTK_TYPE_VBOX,
 		"homogeneous", FALSE,
@@ -231,11 +273,11 @@ xpad_pad_init (XpadPad *pad)
 		"default-height", xpad_settings_get_height (xpad_settings ()),
 		"default-width", xpad_settings_get_width (xpad_settings ()),
 		"gravity", GDK_GRAVITY_STATIC, /* static gravity makes saving pad x,y work */
-		"skip-pager-hint", TRUE,
-		"skip-taskbar-hint", TRUE,
+		"skip-pager-hint", !xpad_settings_get_has_decorations (xpad_settings ()),
+		"skip-taskbar-hint", !xpad_settings_get_has_decorations (xpad_settings ()),
 		"type", GTK_WINDOW_TOPLEVEL,
 		"type-hint", GDK_WINDOW_TYPE_HINT_NORMAL,
-		"window-position", GTK_WIN_POS_NONE,
+		"window-position", GTK_WIN_POS_MOUSE,
 		"child", vbox,
 		NULL);
 	
@@ -269,7 +311,7 @@ xpad_pad_init (XpadPad *pad)
 	g_signal_connect_swapped (pad->priv->toolbar, "activate-delete", G_CALLBACK (xpad_pad_delete), pad);
 	g_signal_connect_swapped (pad->priv->toolbar, "activate-properties", G_CALLBACK (xpad_pad_open_properties), pad);
 	g_signal_connect_swapped (pad->priv->toolbar, "activate-preferences", G_CALLBACK (xpad_pad_open_preferences), pad);
-	g_signal_connect_swapped (pad->priv->toolbar, "activate-quit", G_CALLBACK (xpad_pad_quit), pad);
+	g_signal_connect_swapped (pad->priv->toolbar, "activate-quit", G_CALLBACK (xpad_pad_close_all), pad);
 	
 	g_signal_connect (pad->priv->toolbar, "popup", G_CALLBACK (xpad_pad_toolbar_popup), pad);
 	g_signal_connect (pad->priv->toolbar, "popdown", G_CALLBACK (xpad_pad_toolbar_popdown), pad);
@@ -277,7 +319,7 @@ xpad_pad_init (XpadPad *pad)
 	g_signal_connect (pad->priv->menu, "deactivate", G_CALLBACK (xpad_pad_popup_deactivate), pad);
 	g_signal_connect (pad->priv->highlight_menu, "deactivate", G_CALLBACK (xpad_pad_popup_deactivate), pad);
 	
-	if (xpad_settings_get_sticky (xpad_settings ()))
+	if (pad->priv->sticky)
 		gtk_window_stick (GTK_WINDOW (pad));
 	else
 		gtk_window_unstick (GTK_WINDOW (pad));
@@ -307,14 +349,14 @@ xpad_pad_show (XpadPad *pad)
 		"gravity", GDK_GRAVITY_STATIC,
 		"skip-pager-hint", TRUE,
 		"skip-taskbar-hint", TRUE,
-		NULL);
+		NULL);*/
 	
-	if (xpad_settings_get_sticky (xpad_settings ()))
+	if (pad->priv->sticky)
 		gtk_window_stick (GTK_WINDOW (pad));
 	else
 		gtk_window_unstick (GTK_WINDOW (pad));
 	
-	xpad_pad_sync_title (pad);*/
+/*	xpad_pad_sync_title (pad);*/
 }
 
 static void
@@ -322,6 +364,12 @@ xpad_pad_dispose (GObject *object)
 {
 	XpadPad *pad = XPAD_PAD (object);
 	
+	if (pad->priv->toolbar_timeout)
+	{
+		g_source_remove (pad->priv->toolbar_timeout);
+		pad->priv->toolbar_timeout = 0;
+	}
+	
 	if (pad->priv->properties)
 		gtk_widget_destroy (pad->priv->properties);
 	
@@ -371,21 +419,39 @@ xpad_pad_notify_has_scrollbar (XpadPad *pad)
 static void
 xpad_pad_notify_has_decorations (XpadPad *pad)
 {
-	gboolean shown = GTK_WIDGET_VISIBLE (GTK_WIDGET (pad));
+	gboolean decorations = xpad_settings_get_has_decorations (xpad_settings ());
 	
-	if (shown)
-	{
-		gtk_widget_hide (GTK_WIDGET (pad));
-	}
+	/**
+	 *  There are two modes of operation:  a normal mode and a 'stealth' mode.
+	 *  If decorations are disabled, we also don't show up in the taskbar or pager. 
+	 */
+	gtk_window_set_decorated (GTK_WINDOW (pad), decorations);
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (pad), !decorations);
+	gtk_window_set_skip_pager_hint (GTK_WINDOW (pad), !decorations);
 	
-	gtk_window_set_decorated (GTK_WINDOW (pad), xpad_settings_get_has_decorations (xpad_settings ()));
+	/* reshow_with_initial_size() seems to set the window back to a never-shown state.
+	   This is good, as some WMs don't like us changing the above parameters mid-run,
+	   even if we do a hide/show cycle. */
+	gtk_window_set_default_size (GTK_WINDOW (pad), pad->priv->width, pad->priv->height);
+	gtk_window_reshow_with_initial_size (GTK_WINDOW (pad));
+}
+
+static gint
+xpad_pad_text_and_toolbar_height (XpadPad *pad)
+{
+	GdkRectangle rec;
+	gint textx, texty, x, y;
+	GtkTextIter iter;
 	
-	if (shown)
-	{
-		/* We move it so wm's don't lose the window's spot */
-		gtk_window_move (GTK_WINDOW (pad), pad->priv->x, pad->priv->y);
-		gtk_widget_show (GTK_WIDGET (pad));
-	}
+	gtk_text_view_get_visible_rect(GTK_TEXT_VIEW(pad->priv->textview), &rec);
+	gtk_text_buffer_get_end_iter(gtk_text_view_get_buffer(GTK_TEXT_VIEW(pad->priv->textview)), &iter);
+	gtk_text_view_get_iter_location(GTK_TEXT_VIEW(pad->priv->textview), &iter, &rec);
+	gtk_text_view_buffer_to_window_coords(GTK_TEXT_VIEW(pad->priv->textview),
+		GTK_TEXT_WINDOW_WIDGET, rec.x + rec.width, rec.y + rec.height,
+		&textx, &texty);
+	gtk_widget_translate_coordinates(pad->priv->textview, GTK_WIDGET(pad), textx, texty, &x, &y);
+
+	return y + pad->priv->toolbar_height + gtk_container_get_border_width(GTK_CONTAINER(pad->priv->textview));
 }
 
 static void
@@ -403,8 +469,19 @@ xpad_pad_show_toolbar (XpadPad *pad)
 			gtk_widget_size_request (pad->priv->toolbar, &req);
 			pad->priv->toolbar_height = req.height;
 		}
-		pad->priv->height += pad->priv->toolbar_height;
-		gtk_window_resize (GTK_WINDOW (pad), pad->priv->width, pad->priv->height);
+
+		/* Do we have room for the toolbar without covering text? */
+		if (xpad_pad_text_and_toolbar_height (pad) > pad->priv->height)
+		{
+			pad->priv->toolbar_expanded = TRUE;
+			pad->priv->height += pad->priv->toolbar_height;
+			gtk_window_resize (GTK_WINDOW (pad), pad->priv->width, pad->priv->height);
+		}
+		else
+			pad->priv->toolbar_expanded = FALSE;
+		
+		pad->priv->toolbar_pad_resized = FALSE;
+		
 		if (GTK_WIDGET (pad)->window)
 			gdk_window_thaw_updates (GTK_WIDGET (pad)->window);
 	}
@@ -417,9 +494,15 @@ xpad_pad_hide_toolbar (XpadPad *pad)
 	{
 		if (GTK_WIDGET (pad)->window)
 			gdk_window_freeze_updates (GTK_WIDGET (pad)->window);
-		pad->priv->height -= pad->priv->toolbar_height;
 		gtk_widget_hide (pad->priv->toolbar);
-		gtk_window_resize (GTK_WINDOW (pad), pad->priv->width, pad->priv->height);
+		
+		if (pad->priv->toolbar_expanded ||
+		    (pad->priv->toolbar_pad_resized && xpad_pad_text_and_toolbar_height (pad) >= pad->priv->height))
+		{
+				pad->priv->height -= pad->priv->toolbar_height;
+				gtk_window_resize (GTK_WINDOW (pad), pad->priv->width, pad->priv->height);
+				pad->priv->toolbar_expanded = FALSE;
+		}
 		if (GTK_WIDGET (pad)->window)
 			gdk_window_thaw_updates (GTK_WIDGET (pad)->window);
 	}
@@ -510,11 +593,21 @@ xpad_pad_clear (XpadPad *pad)
 	gtk_text_buffer_set_text (buffer, "", -1);
 }
 
-static void
+void
 xpad_pad_close (XpadPad *pad)
 {
 	gtk_widget_hide (GTK_WIDGET (pad));
 	
+	/* If no tray and this is the last pad, we don't want to record this
+	   pad as closed, we want to start with just this pad next open.  So
+	   quit before we record. */
+	if (!xpad_tray_is_open () &&
+	    xpad_pad_group_num_visible_pads (pad->priv->group) == 0)
+	{
+		xpad_pad_quit (pad);
+		return;
+	}
+	
 	if (pad->priv->properties)
 		gtk_widget_destroy (pad->priv->properties);
 	
@@ -748,6 +841,12 @@ xpad_pad_toolbar_size_allocate (XpadPad *pad, GtkAllocation *event)
 static gboolean
 xpad_pad_configure_event (XpadPad *pad, GdkEventConfigure *event)
 {
+	if (!GTK_WIDGET_VISIBLE (pad))
+		return FALSE;
+	
+	if (pad->priv->width != event->width || pad->priv->height != event->height)
+		pad->priv->toolbar_pad_resized = TRUE;
+	
 	pad->priv->x = event->x;
 	pad->priv->y = event->y;
 	pad->priv->width = event->width;
@@ -771,10 +870,13 @@ xpad_pad_configure_event (XpadPad *pad, GdkEventConfigure *event)
 static gboolean
 xpad_pad_window_state_event (XpadPad *pad, GdkEventWindowState *event)
 {
-	pad->priv->sticky = (event->new_window_state & GDK_WINDOW_STATE_STICKY) ? TRUE : FALSE;
-	
-	if (event->changed_mask & GDK_WINDOW_STATE_STICKY)
-		save_info (pad);
+	if (event->changed_mask & GDK_WINDOW_STATE_STICKY) {
+		if (GTK_WIDGET_VISIBLE (pad))
+		{
+			pad->priv->sticky = (event->new_window_state & GDK_WINDOW_STATE_STICKY) ? TRUE : FALSE;
+			save_info (pad);
+		}
+	}
 	
 	return FALSE;
 }
@@ -950,9 +1052,14 @@ load_content (XpadPad *pad)
 	content = fio_get_file (pad->priv->contentname);
 	
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (pad->priv->textview));
-	xpad_text_buffer_set_text_with_tags (XPAD_TEXT_BUFFER (buffer), content ? content : "");
 	
+	g_signal_handlers_block_by_func (buffer, xpad_pad_text_changed, pad);
+	
+	xpad_text_buffer_set_text_with_tags (XPAD_TEXT_BUFFER (buffer), content ? content : "");
 	g_free (content);
+	
+	g_signal_handlers_unblock_by_func (buffer, xpad_pad_text_changed, pad);
+	xpad_pad_text_changed(pad, buffer);
 }
 
 static void
@@ -978,7 +1085,7 @@ save_content (XpadPad *pad)
 }
 
 static void
-load_info (XpadPad *pad)
+load_info (XpadPad *pad, gboolean *show)
 {
 	gboolean locked = FALSE, follow_font = TRUE, follow_color = TRUE;
 	gboolean hidden = FALSE;
@@ -1063,8 +1170,8 @@ load_info (XpadPad *pad)
 	}
 	g_free (oldcontentprefix);
 	
-	if (hidden)
-		gtk_widget_hide (GTK_WIDGET (pad));
+	if (show)
+		*show = !hidden;
 }
 
 static void
@@ -1080,6 +1187,7 @@ save_info (XpadPad *pad)
 		pad->priv->infoname = fio_unique_name ("info-");
 		if (!pad->priv->infoname)
 			return;
+		gtk_window_set_role (GTK_WINDOW (pad), pad->priv->infoname);
 	}
 	/* create content file if it doesn't exist yet */
 	if (!pad->priv->contentname)
@@ -1090,7 +1198,7 @@ save_info (XpadPad *pad)
 	}
 	
 	height = pad->priv->height;
-	if (GTK_WIDGET_VISIBLE (pad->priv->toolbar))
+	if (GTK_WIDGET_VISIBLE (pad->priv->toolbar) && pad->priv->toolbar_expanded)
 		height -= pad->priv->toolbar_height;
 	
 	style = gtk_widget_get_style (pad->priv->textview);
@@ -1104,7 +1212,7 @@ save_info (XpadPad *pad)
 		"b|follow_font", xpad_text_view_get_follow_font_style (XPAD_TEXT_VIEW (pad->priv->textview)),
 		"b|follow_color", xpad_text_view_get_follow_color_style (XPAD_TEXT_VIEW (pad->priv->textview)),
 		"b|sticky", pad->priv->sticky,
-		"b|hidden", GTK_WIDGET_VISIBLE (pad),
+		"b|hidden", !GTK_WIDGET_VISIBLE (pad),
 		"h|back_red", style->base[GTK_STATE_NORMAL].red,
 		"h|back_green", style->base[GTK_STATE_NORMAL].green,
 		"h|back_blue", style->base[GTK_STATE_NORMAL].blue,
@@ -1124,7 +1232,7 @@ menu_about (XpadPad *pad)
 	const gchar *artists[] = {"Michael Terry <mike@mterry.name>", NULL};
 	const gchar *authors[] = {"Michael Terry <mike@mterry.name>", "Jeroen Vermeulen <jtv@xs4all.nl>", NULL};
 	const gchar *comments = _("Sticky notes");
-	const gchar *copyright = "© 2001-2005 Michael Terry";
+	const gchar *copyright = "© 2001-2007 Michael Terry";
 	/* we use g_strdup_printf because C89 has size limits on static strings */
 	gchar *license = g_strdup_printf ("%s\n%s\n%s",
 "This program is free software; you can redistribute it and/or\n"
@@ -1207,6 +1315,24 @@ menu_show_all (XpadPad *pad)
 	g_slist_free (pads);
 }
 
+static void
+xpad_pad_close_all (XpadPad *pad)
+{
+	if (!pad->priv->group)
+		return;
+	
+	/**
+	 * The logic is different here depending on whether the tray is open.
+	 * If it is open, we just close each pad individually.  If it isn't
+	 * open, we do a quit.  This way, when xpad is run again, only the
+	 * pads open during the last 'close all' will open again.
+	 */
+	if (xpad_tray_is_open ())
+		xpad_pad_group_close_all (pad->priv->group);
+	else
+		xpad_pad_quit (pad);
+}
+
 static void
 menu_show (XpadPad *pad)
 {
@@ -1373,6 +1499,7 @@ menu_get_popup_no_highlight (XpadPad *pad, GtkAccelGroup *accel_group)
 	MENU_ADD_STOCK (GTK_STOCK_NEW, xpad_pad_spawn);
 	MENU_ADD_SEP ();
 	MENU_ADD_CHECK (_("Show on _All Workspaces"), pad->priv->sticky, menu_sticky);
+	g_object_set_data (G_OBJECT (uppermenu), "sticky", item);
 	MENU_ADD_STOCK (GTK_STOCK_PROPERTIES, xpad_pad_open_properties);
 	MENU_ADD_SEP ();
 	MENU_ADD_STOCK (GTK_STOCK_CLOSE, xpad_pad_close);
@@ -1415,7 +1542,7 @@ menu_get_popup_no_highlight (XpadPad *pad, GtkAccelGroup *accel_group)
 	g_object_set_data (G_OBJECT (uppermenu), "notes-menu", menu);
 	
 	MENU_ADD (_("_Show All"), NULL, 0, 0, menu_show_all);
-	MENU_ADD (_("_Close All"), GTK_STOCK_QUIT, GDK_q, GDK_CONTROL_MASK, xpad_pad_quit);
+	MENU_ADD (_("_Close All"), NULL, 0, 0, xpad_pad_close_all);
 	
 	/* The rest of the notes menu will get set up in the prep function below */
 	
@@ -1444,6 +1571,13 @@ menu_prep_popup_no_highlight (XpadPad *current_pad, GtkWidget *uppermenu)
 	if (item)
 		gtk_widget_set_sensitive (item, gtk_clipboard_wait_is_text_available (clipboard));
 	
+	item = g_object_get_data (G_OBJECT (uppermenu), "sticky");
+	if (item) {
+		g_signal_handlers_block_by_func (item, menu_sticky, current_pad);
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), current_pad->priv->sticky);
+		g_signal_handlers_unblock_by_func (item, menu_sticky, current_pad);
+	}
+	
 	menu = g_object_get_data (G_OBJECT (uppermenu), "notes-menu");
 	if (menu)
 	{
