@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2001-2004 Michael Terry
+Copyright (c) 2001-2007 Michael Terry
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -242,7 +242,7 @@ xpad_settings_init (XpadSettings *settings)
 	settings->priv->fontname = NULL;
 	settings->priv->has_toolbar = TRUE;
 	settings->priv->autohide_toolbar = TRUE;
-	settings->priv->has_scrollbar = FALSE;
+	settings->priv->has_scrollbar = TRUE;
 	
 	settings->priv->toolbar_buttons = NULL;
 	settings->priv->toolbar_buttons = g_slist_append (settings->priv->toolbar_buttons, g_strdup ("New"));
