@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2001-2004 Michael Terry
+Copyright (c) 2001-2007 Michael Terry
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -37,8 +37,6 @@ enum {
 
 struct XpadToolbarPrivate
 {
-	GtkTooltips *tooltips;
-	
 	GtkToolItem *move_button;
 	gboolean move_removed;
 	guint move_index;
@@ -79,7 +77,7 @@ static const XpadToolbarButton buttons[] =
 	{"New", "gtk-new", ACTIVATE_NEW, XPAD_BUTTON_TYPE_BUTTON, N_("Open New Pad"), N_("Add _New to Toolbar")},
 	{"Preferences", "gtk-preferences", ACTIVATE_PREFERENCES, XPAD_BUTTON_TYPE_BUTTON, N_("Edit Preferences"), N_("Add Pr_eferences to Toolbar")},
 	{"Properties", "gtk-properties", ACTIVATE_PROPERTIES, XPAD_BUTTON_TYPE_BUTTON, N_("Edit Pad Properties"), N_("Add Proper_ties to Toolbar")},
-	{"Quit", "gtk-quit", ACTIVATE_QUIT, XPAD_BUTTON_TYPE_BUTTON, N_("Close All Pads"), N_("Add _Quit to Toolbar")},
+	{"Quit", "gtk-quit", ACTIVATE_QUIT, XPAD_BUTTON_TYPE_BUTTON, N_("Close All Pads"), N_("Add Close _All to Toolbar")},
 	{"sep", NULL, 0, XPAD_BUTTON_TYPE_SEPARATOR, NULL, N_("Add a Se_parator to Toolbar")} /* Separator */
 	/*{"Minimize to Tray", "gtk-goto-bottom", 1, N_("Minimize Pads to System Tray")}*/
 };
@@ -202,18 +200,15 @@ xpad_toolbar_init (XpadToolbar *toolbar)
 {
 	toolbar->priv = XPAD_TOOLBAR_GET_PRIVATE (toolbar);
 	
-	toolbar->priv->tooltips = gtk_tooltips_new ();
-	g_object_ref (toolbar->priv->tooltips);
-	gtk_object_sink (GTK_OBJECT (toolbar->priv->tooltips));
-	
 	toolbar->priv->move_motion_handler = 0;
 	toolbar->priv->move_button_release_handler = 0;
 	toolbar->priv->move_key_press_handler = 0;
 	
-	gtk_toolbar_set_tooltips (GTK_TOOLBAR (toolbar), TRUE);
-	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
-	gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), FALSE);
-	gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_MENU);
+	g_object_set (G_OBJECT (toolbar),
+	              "icon-size", GTK_ICON_SIZE_MENU,
+	              "show-arrow", FALSE,
+	              "toolbar-style", GTK_TOOLBAR_ICONS,
+	              NULL);
 	
 	g_signal_connect_swapped (xpad_settings (), "change-buttons", G_CALLBACK (xpad_toolbar_change_buttons), toolbar);
 	
@@ -225,8 +220,6 @@ xpad_toolbar_finalize (GObject *object)
 {
 	XpadToolbar *toolbar = XPAD_TOOLBAR (object);
 	
-	g_object_unref (toolbar->priv->tooltips);
-	
 	if (toolbar->priv->move_button)
 		g_object_unref (toolbar->priv->move_button);
 	
@@ -293,7 +286,7 @@ xpad_toolbar_button_to_item (XpadToolbar *toolbar, const XpadToolbarButton *butt
 	g_object_set_data (G_OBJECT (item), "xpad-tb", (gpointer) button);
 	
 	if (button->desc)
-		gtk_tool_item_set_tooltip (item, toolbar->priv->tooltips, _(button->desc), _(button->desc));
+		gtk_tool_item_set_tooltip_text (item, _(button->desc));
 	
 	child = gtk_bin_get_child (GTK_BIN (item));
 	if (child)
